@@ -25,6 +25,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.TreeMap;
+import java.util.function.Supplier;
 
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
@@ -65,10 +66,13 @@ import org.apache.uniffle.client.response.CompressedShuffleBlock;
 import org.apache.uniffle.client.response.SendShuffleDataResult;
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
-import org.apache.uniffle.common.RssShuffleUtils;
 import org.apache.uniffle.common.ShuffleAssignmentsInfo;
 import org.apache.uniffle.common.ShuffleBlockInfo;
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleServerInfo;
+import org.apache.uniffle.common.compression.Codec;
+import org.apache.uniffle.common.compression.Lz4Codec;
+import org.apache.uniffle.common.config.RssConf;
 import org.apache.uniffle.common.exception.RssException;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -88,6 +92,8 @@ public class FetcherTest {
   static List<byte[]> data;
   static MergeManagerImpl<Text, Text> merger;
 
+  static Codec codec = new Lz4Codec();
+
   @Test
   public void writeAndReadDataTestWithRss() throws Throwable {
     fs = FileSystem.getLocal(conf);
@@ -97,7 +103,7 @@ public class FetcherTest {
         null, null, new Progress(), new MROutputFiles());
     ShuffleReadClient shuffleReadClient = new MockedShuffleReadClient(data);
     RssFetcher fetcher = new RssFetcher(jobConf, reduceId1, taskStatus, merger, new Progress(),
-        reporter, metrics, shuffleReadClient, 3);
+        reporter, metrics, shuffleReadClient, 3, new RssConf());
     fetcher.fetchAllRssBlocks();
 
 
@@ -115,8 +121,8 @@ public class FetcherTest {
       allKeys.add(new Text(key).toString().trim());
       allValues.add(new Text(value).toString().trim());
     }
-    validate(allKeysExpected, allKeys);
-    validate(allValuesExpected, allValues);
+    assertEquals(allKeysExpected, allKeys);
+    assertEquals(allValuesExpected, allValues);
   }
 
   @Test
@@ -128,7 +134,7 @@ public class FetcherTest {
         null, null, new Progress(), new MROutputFiles());
     ShuffleReadClient shuffleReadClient = new MockedShuffleReadClient(data);
     RssFetcher fetcher = new RssFetcher(jobConf, reduceId1, taskStatus, merger, new Progress(),
-        reporter, metrics, shuffleReadClient, 3);
+        reporter, metrics, shuffleReadClient, 3, new RssConf());
     fetcher.fetchAllRssBlocks();
 
 
@@ -146,8 +152,8 @@ public class FetcherTest {
       allKeys.add(new Text(key).toString().trim());
       allValues.add(new Text(value).toString().trim());
     }
-    validate(allKeysExpected, allKeys);
-    validate(allValuesExpected, allValues);
+    assertEquals(allKeysExpected, allKeys);
+    assertEquals(allValuesExpected, allValues);
   }
 
   @Test
@@ -161,7 +167,7 @@ public class FetcherTest {
       null, null, new Progress(), new MROutputFiles(), expectedFails);
     ShuffleReadClient shuffleReadClient = new MockedShuffleReadClient(data);
     RssFetcher fetcher = new RssFetcher(jobConf, reduceId1, taskStatus, merger, new Progress(),
-        reporter, metrics, shuffleReadClient, 3);
+        reporter, metrics, shuffleReadClient, 3, new RssConf());
     fetcher.fetchAllRssBlocks();
 
     RawKeyValueIterator iterator = merger.close();
@@ -178,11 +184,11 @@ public class FetcherTest {
       allKeys.add(new Text(key).toString().trim());
       allValues.add(new Text(value).toString().trim());
     }
-    validate(allKeysExpected, allKeys);
-    validate(allValuesExpected, allValues);
+    assertEquals(allKeysExpected, allKeys);
+    assertEquals(allValuesExpected, allValues);
     // There will be 2 retries
-    assert (fetcher.getRetryCount() == 2);
-    assert (((MockMergeManagerImpl)merger).happenedFails.size() == 2);
+    assertEquals(2, fetcher.getRetryCount());
+    assertEquals(2, ((MockMergeManagerImpl)merger).happenedFails.size());
   }
 
   public void testCodecIsDuplicated() throws Exception {
@@ -202,13 +208,6 @@ public class FetcherTest {
         (InMemoryMapOutput) mapOutput1), RssBypassWriter.getDecompressor((InMemoryMapOutput) mapOutput2));
   }
 
-  private void validate(List<String> expected, List<String> actual) {
-    assert (expected.size() == actual.size());
-    for (int i = 0; i < expected.size(); i++) {
-      assert (expected.get(i).equals(actual.get(i)));
-    }
-  }
-
   private static void initRssData() throws Exception {
     data = new LinkedList<>();
     Map<String, String> map1 = new TreeMap<>();
@@ -276,7 +275,8 @@ public class FetcherTest {
         true,
         5,
         0.2f,
-        1024000L);
+        1024000L,
+        new RssConf());
 
     for (String key : keysToValues.keySet()) {
       String value = keysToValues.get(key);
@@ -346,7 +346,8 @@ public class FetcherTest {
     public List<byte[]> data = new LinkedList<>();
 
     @Override
-    public SendShuffleDataResult sendShuffleData(String appId, List<ShuffleBlockInfo> shuffleBlockInfoList) {
+    public SendShuffleDataResult sendShuffleData(String appId, List<ShuffleBlockInfo> shuffleBlockInfoList,
+        Supplier<Boolean> needCancelRequest) {
       if (mode == 0) {
         throw new RssException("send data failed");
       } else if (mode == 1) {
@@ -357,7 +358,14 @@ public class FetcherTest {
           successBlockIds.add(blockInfo.getBlockId());
         }
         shuffleBlockInfoList.forEach(block -> {
-          data.add(RssShuffleUtils.decompressData(block.getData(), block.getUncompressLength()));
+          ByteBuffer uncompressedBuffer = ByteBuffer.allocate(block.getUncompressLength());
+          codec.decompress(
+              ByteBuffer.wrap(block.getData()),
+              block.getUncompressLength(),
+              uncompressedBuffer,
+              0
+          );
+          data.add(uncompressedBuffer.array());
         });
         return new SendShuffleDataResult(successBlockIds, Sets.newHashSet());
       }
@@ -368,13 +376,19 @@ public class FetcherTest {
 
     }
 
+    @Override
+    public void registerApplicationInfo(String appId, long timeoutMs, String user) {
+
+    }
+
     @Override
     public void registerShuffle(
         ShuffleServerInfo shuffleServerInfo,
         String appId,
         int shuffleId,
         List<PartitionRange> partitionRanges,
-        RemoteStorageInfo storageType) {
+        RemoteStorageInfo storageType,
+        ShuffleDataDistributionType distributionType) {
 
     }
 
@@ -406,7 +420,8 @@ public class FetcherTest {
 
     @Override
     public ShuffleAssignmentsInfo getShuffleAssignments(String appId, int shuffleId, int partitionNum,
-        int partitionNumPerRange, Set<String> requiredTags, int assignmentShuffleServerNumber) {
+        int partitionNumPerRange, Set<String> requiredTags, int assignmentShuffleServerNumber,
+        int estimateTaskConcurrency) {
       return null;
     }
 
@@ -426,6 +441,11 @@ public class FetcherTest {
     public void close() {
 
     }
+
+    @Override
+    public void unregisterShuffle(String appId, int shuffleId) {
+
+    }
   }
 
   static class MockedShuffleReadClient implements ShuffleReadClient {
@@ -435,7 +455,7 @@ public class FetcherTest {
     MockedShuffleReadClient(List<byte[]> data) {
       this.blocks = new LinkedList<>();
       data.forEach(bytes -> {
-        byte[] compressed = RssShuffleUtils.compressData(bytes);
+        byte[] compressed = codec.compress(bytes);
         blocks.add(new CompressedShuffleBlock(ByteBuffer.wrap(compressed), bytes.length));
       });
     }
