@@ -20,7 +20,6 @@ package org.apache.spark.shuffle.reader;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 
-import com.esotericsoftware.kryo.io.Input;
 import com.google.common.annotations.VisibleForTesting;
 import io.netty.buffer.ByteBufInputStream;
 import io.netty.buffer.Unpooled;
@@ -28,6 +27,7 @@ import org.apache.spark.executor.ShuffleReadMetrics;
 import org.apache.spark.serializer.DeserializationStream;
 import org.apache.spark.serializer.Serializer;
 import org.apache.spark.serializer.SerializerInstance;
+import org.apache.spark.shuffle.RssSparkConfig;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import scala.Product2;
@@ -38,8 +38,8 @@ import scala.runtime.BoxedUnit;
 
 import org.apache.uniffle.client.api.ShuffleReadClient;
 import org.apache.uniffle.client.response.CompressedShuffleBlock;
-import org.apache.uniffle.common.RssShuffleUtils;
-import org.apache.uniffle.common.exception.RssException;
+import org.apache.uniffle.common.compression.Codec;
+import org.apache.uniffle.common.config.RssConf;
 
 public class RssShuffleDataIterator<K, C> extends AbstractIterator<Product2<K, C>> {
 
@@ -52,25 +52,30 @@ public class RssShuffleDataIterator<K, C> extends AbstractIterator<Product2<K, C
   private long readTime = 0;
   private long serializeTime = 0;
   private long decompressTime = 0;
-  private Input deserializationInput = null;
   private DeserializationStream deserializationStream = null;
   private ByteBufInputStream byteBufInputStream = null;
-  private long compressedBytesLength = 0;
+  private long totalRawBytesLength = 0;
   private long unCompressedBytesLength = 0;
   private ByteBuffer uncompressedData;
+  private Codec codec;
 
   public RssShuffleDataIterator(
       Serializer serializer,
       ShuffleReadClient shuffleReadClient,
-      ShuffleReadMetrics shuffleReadMetrics) {
+      ShuffleReadMetrics shuffleReadMetrics,
+      RssConf rssConf) {
     this.serializerInstance = serializer.newInstance();
     this.shuffleReadClient = shuffleReadClient;
     this.shuffleReadMetrics = shuffleReadMetrics;
+    boolean compress = rssConf.getBoolean(RssSparkConfig.SPARK_SHUFFLE_COMPRESS_KEY
+            .substring(RssSparkConfig.SPARK_RSS_CONFIG_PREFIX.length()),
+        RssSparkConfig.SPARK_SHUFFLE_COMPRESS_DEFAULT);
+    this.codec = compress ? Codec.newInstance(rssConf) : null;
   }
 
-  public Iterator<Tuple2<Object, Object>> createKVIterator(ByteBuffer data) {
+  public Iterator<Tuple2<Object, Object>> createKVIterator(ByteBuffer data, int size) {
     clearDeserializationStream();
-    byteBufInputStream = new ByteBufInputStream(Unpooled.wrappedBuffer(data), true);
+    byteBufInputStream = new ByteBufInputStream(Unpooled.wrappedBuffer(data.array(), data.position(), size), true);
     deserializationStream = serializerInstance.deserializeStream(byteBufInputStream);
     return deserializationStream.asKeyValueIterator();
   }
@@ -83,13 +88,9 @@ public class RssShuffleDataIterator<K, C> extends AbstractIterator<Product2<K, C
         LOG.warn("Can't close ByteBufInputStream, memory may be leaked.");
       }
     }
-    if (deserializationInput != null) {
-      deserializationInput.close();
-    }
     if (deserializationStream != null) {
       deserializationStream.close();
     }
-    deserializationInput = null;
     deserializationStream = null;
     byteBufInputStream = null;
   }
@@ -99,37 +100,18 @@ public class RssShuffleDataIterator<K, C> extends AbstractIterator<Product2<K, C
     if (recordsIterator == null || !recordsIterator.hasNext()) {
       // read next segment
       long startFetch = System.currentTimeMillis();
-      CompressedShuffleBlock compressedBlock = shuffleReadClient.readShuffleBlockData();
+      // depends on spark.shuffle.compress, shuffled block may not be compressed
+      CompressedShuffleBlock rawBlock = shuffleReadClient.readShuffleBlockData();
       // If ShuffleServer delete
 
-      ByteBuffer compressedData = null;
-      if (compressedBlock != null) {
-        compressedData = compressedBlock.getByteBuffer();
-      }
+      ByteBuffer rawData = rawBlock != null ? rawBlock.getByteBuffer() : null;
       long fetchDuration = System.currentTimeMillis() - startFetch;
       shuffleReadMetrics.incFetchWaitTime(fetchDuration);
-      if (compressedData != null) {
-        long compressedDataLength = compressedData.limit() - compressedData.position();
-        compressedBytesLength += compressedDataLength;
-        shuffleReadMetrics.incRemoteBytesRead(compressedDataLength);
-        // Directbytebuffers are not collected in time will cause executor easy
-        // be killed by cluster managers(such as YARN) for using too much offheap memory
-        if (uncompressedData != null && uncompressedData.isDirect()) {
-          try {
-            RssShuffleUtils.destroyDirectByteBuffer(uncompressedData);
-          } catch (Exception e) {
-            throw new RssException("Destroy DirectByteBuffer failed!", e);
-          }
-        }
-        long startDecompress = System.currentTimeMillis();
-        uncompressedData = RssShuffleUtils.decompressData(
-            compressedData, compressedBlock.getUncompressLength());
-        unCompressedBytesLength += compressedBlock.getUncompressLength();
-        long decompressDuration = System.currentTimeMillis() - startDecompress;
-        decompressTime += decompressDuration;
+      if (rawData != null) {
+        int uncompressedLen = uncompress(rawBlock, rawData);
         // create new iterator for shuffle data
         long startSerialization = System.currentTimeMillis();
-        recordsIterator = createKVIterator(uncompressedData);
+        recordsIterator = createKVIterator(uncompressedData, uncompressedLen);
         long serializationDuration = System.currentTimeMillis() - startSerialization;
         readTime += fetchDuration;
         serializeTime += serializationDuration;
@@ -137,15 +119,40 @@ public class RssShuffleDataIterator<K, C> extends AbstractIterator<Product2<K, C
         // finish reading records, check data consistent
         shuffleReadClient.checkProcessedBlockIds();
         shuffleReadClient.logStatics();
-        LOG.info("Fetch " + compressedBytesLength + " bytes cost " + readTime + " ms and "
-            + serializeTime + " ms to serialize, " + decompressTime + " ms to decompress with unCompressionLength["
+        String decInfo = codec == null ? "." : (", " + decompressTime
+            + " ms to decompress with unCompressionLength["
             + unCompressedBytesLength + "]");
+        LOG.info("Fetch {} bytes cost {} ms and {} ms to serialize{}",
+            totalRawBytesLength, readTime, serializeTime, decInfo);
         return false;
       }
     }
     return recordsIterator.hasNext();
   }
 
+  private int uncompress(CompressedShuffleBlock rawBlock, ByteBuffer rawData) {
+    long rawDataLength = rawData.limit() - rawData.position();
+    totalRawBytesLength += rawDataLength;
+    shuffleReadMetrics.incRemoteBytesRead(rawDataLength);
+
+    int uncompressedLen = rawBlock.getUncompressLength();
+    if (codec != null) {
+      if (uncompressedData == null || uncompressedData.capacity() < uncompressedLen) {
+        // todo: support off-heap bytebuffer
+        uncompressedData = ByteBuffer.allocate(uncompressedLen);
+      }
+      uncompressedData.clear();
+      long startDecompress = System.currentTimeMillis();
+      codec.decompress(rawData, uncompressedLen, uncompressedData, 0);
+      unCompressedBytesLength += uncompressedLen;
+      long decompressDuration = System.currentTimeMillis() - startDecompress;
+      decompressTime += decompressDuration;
+    } else {
+      uncompressedData = rawData;
+    }
+    return uncompressedLen;
+  }
+
   @Override
   public Product2<K, C> next() {
     shuffleReadMetrics.incRecordsRead(1L);
@@ -158,6 +165,7 @@ public class RssShuffleDataIterator<K, C> extends AbstractIterator<Product2<K, C
       shuffleReadClient.close();
     }
     shuffleReadClient = null;
+    uncompressedData = null;
     return BoxedUnit.UNIT;
   }
 
