@@ -49,7 +49,9 @@ import scala.runtime.BoxedUnit;
 import org.apache.uniffle.client.api.ShuffleReadClient;
 import org.apache.uniffle.client.factory.ShuffleClientFactory;
 import org.apache.uniffle.client.request.CreateShuffleReadClientRequest;
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleServerInfo;
+import org.apache.uniffle.common.config.RssConf;
 
 public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
   private static final Logger LOG = LoggerFactory.getLogger(RssShuffleReader.class);
@@ -74,6 +76,8 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
   private int mapStartIndex;
   private int mapEndIndex;
   private ShuffleReadMetrics readMetrics;
+  private RssConf rssConf;
+  private ShuffleDataDistributionType dataDistributionType;
 
   public RssShuffleReader(
       int startPartition,
@@ -90,7 +94,9 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
       int partitionNum,
       Map<Integer, Roaring64NavigableMap> partitionToExpectBlocks,
       Roaring64NavigableMap taskIdBitmap,
-      ShuffleReadMetrics readMetrics) {
+      ShuffleReadMetrics readMetrics,
+      RssConf rssConf,
+      ShuffleDataDistributionType dataDistributionType) {
     this.appId = rssShuffleHandle.getAppId();
     this.startPartition = startPartition;
     this.endPartition = endPartition;
@@ -111,6 +117,8 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
     this.hadoopConf = hadoopConf;
     this.readMetrics = readMetrics;
     this.partitionToShuffleServers = rssShuffleHandle.getPartitionToServers();
+    this.rssConf = rssConf;
+    this.dataDistributionType = dataDistributionType;
   }
 
   @Override
@@ -153,7 +161,7 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
       Function1<TaskContext, Void> fn1 = new AbstractFunction1<TaskContext, Void>() {
         public Void apply(TaskContext context) {
           sorter.stop();
-          return (Void) null;
+          return null;
         }
       };
       context.addTaskCompletionListener(fn1);
@@ -195,13 +203,18 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
           continue;
         }
         List<ShuffleServerInfo> shuffleServerInfoList = partitionToShuffleServers.get(partition);
+        // This mechanism of expectedTaskIdsBitmap filter is to filter out the most of data.
+        // especially for AQE skew optimization
+        boolean expectedTaskIdsBitmapFilterEnable = !(mapStartIndex == 0 && mapEndIndex == Integer.MAX_VALUE)
+            || shuffleServerInfoList.size() > 1;
         CreateShuffleReadClientRequest request = new CreateShuffleReadClientRequest(
             appId, shuffleId, partition, storageType, basePath, indexReadLimit, readBufferSize,
-            1, partitionNum, partitionToExpectBlocks.get(partition), taskIdBitmap, shuffleServerInfoList, hadoopConf);
+            1, partitionNum, partitionToExpectBlocks.get(partition), taskIdBitmap, shuffleServerInfoList,
+            hadoopConf, dataDistributionType, expectedTaskIdsBitmapFilterEnable);
         ShuffleReadClient shuffleReadClient = ShuffleClientFactory.getInstance().createShuffleReadClient(request);
         RssShuffleDataIterator iterator = new RssShuffleDataIterator<K, C>(
             shuffleDependency.serializer(), shuffleReadClient,
-            readMetrics);
+            readMetrics, rssConf);
         CompletionIterator<Product2<K, C>, RssShuffleDataIterator<K, C>> completionIterator =
             CompletionIterator$.MODULE$.apply(iterator, () -> {
               context.taskMetrics().mergeShuffleReadMetrics();
@@ -218,7 +231,7 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
         if (dataIterator != null) {
           dataIterator.completion();
         }
-        iterator.forEachRemaining(ci -> ci.completion());
+        iterator.forEachRemaining(CompletionIterator::completion);
       });
     }
 
