@@ -30,9 +30,8 @@ import java.net.Inet4Address;
 import java.net.InetAddress;
 import java.net.InterfaceAddress;
 import java.net.NetworkInterface;
-import java.nio.BufferUnderflowException;
-import java.nio.ByteBuffer;
 import java.nio.charset.StandardCharsets;
+import java.util.Arrays;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -49,15 +48,15 @@ import org.roaringbitmap.longlong.Roaring64NavigableMap;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import org.apache.uniffle.common.BufferSegment;
-import org.apache.uniffle.common.ShuffleDataSegment;
-import org.apache.uniffle.common.ShuffleIndexResult;
 import org.apache.uniffle.common.ShuffleServerInfo;
+import org.apache.uniffle.common.config.RssBaseConf;
+import org.apache.uniffle.common.config.RssConf;
 import org.apache.uniffle.common.exception.RssException;
 
 public class RssUtils {
 
   private static final Logger LOGGER = LoggerFactory.getLogger(RssUtils.class);
+  public static final String RSS_LOCAL_DIR_KEY = "RSS_LOCAL_DIRS";
 
   private RssUtils() {
   }
@@ -184,71 +183,6 @@ public class RssUtils {
     return clone;
   }
 
-  public static List<ShuffleDataSegment> transIndexDataToSegments(
-      ShuffleIndexResult shuffleIndexResult, int readBufferSize) {
-    if (shuffleIndexResult == null || shuffleIndexResult.isEmpty()) {
-      return Lists.newArrayList();
-    }
-
-    byte[] indexData = shuffleIndexResult.getIndexData();
-    long dataFileLen = shuffleIndexResult.getDataFileLen();
-    return transIndexDataToSegments(indexData, readBufferSize, dataFileLen);
-  }
-
-  private static List<ShuffleDataSegment> transIndexDataToSegments(byte[] indexData,
-      int readBufferSize, long dataFileLen) {
-    ByteBuffer byteBuffer = ByteBuffer.wrap(indexData);
-    List<BufferSegment> bufferSegments = Lists.newArrayList();
-    List<ShuffleDataSegment> dataFileSegments = Lists.newArrayList();
-    int bufferOffset = 0;
-    long fileOffset = -1;
-    long totalLength = 0;
-
-    while (byteBuffer.hasRemaining()) {
-      try {
-        long offset = byteBuffer.getLong();
-        int length = byteBuffer.getInt();
-        int uncompressLength = byteBuffer.getInt();
-        long crc = byteBuffer.getLong();
-        long blockId = byteBuffer.getLong();
-        long taskAttemptId = byteBuffer.getLong();
-        // The index file is written, read and parsed sequentially, so these parsed index segments
-        // index a continuous shuffle data in the corresponding data file and the first segment's
-        // offset field is the offset of these shuffle data in the data file.
-        if (fileOffset == -1) {
-          fileOffset = offset;
-        }
-
-        bufferSegments.add(new BufferSegment(blockId, bufferOffset, length, uncompressLength, crc, taskAttemptId));
-        bufferOffset += length;
-        totalLength += length;
-
-        // If ShuffleServer is flushing the file at this time, the length in the index file record may be greater
-        // than the length in the actual data file, and it needs to be returned at this time to avoid EOFException
-        if (dataFileLen != -1 && totalLength >= dataFileLen) {
-          break;
-        }
-
-        if (bufferOffset >= readBufferSize) {
-          ShuffleDataSegment sds = new ShuffleDataSegment(fileOffset, bufferOffset, bufferSegments);
-          dataFileSegments.add(sds);
-          bufferSegments = Lists.newArrayList();
-          bufferOffset = 0;
-          fileOffset = -1;
-        }
-      } catch (BufferUnderflowException ue) {
-        throw new RssException("Read index data under flow", ue);
-      }
-    }
-
-    if (bufferOffset > 0) {
-      ShuffleDataSegment sds = new ShuffleDataSegment(fileOffset, bufferOffset, bufferSegments);
-      dataFileSegments.add(sds);
-    }
-
-    return dataFileSegments;
-  }
-
   public static String generateShuffleKey(String appId, int shuffleId) {
     return String.join(Constants.KEY_SPLIT_CHAR, appId, String.valueOf(shuffleId));
   }
@@ -257,14 +191,14 @@ public class RssUtils {
     return String.join(Constants.KEY_SPLIT_CHAR, appId, String.valueOf(shuffleId), String.valueOf(partition));
   }
 
-  public static <T> List<T> loadExtensions(
-      Class<T> extClass, List<String> classes, Object obj) throws RuntimeException {
+  public static <T> List<T> loadExtensions(Class<T> extClass, List<String> classes, Object obj) {
     if (classes == null || classes.isEmpty()) {
       throw new RuntimeException("Empty classes");
     }
 
     List<T> extensions = Lists.newArrayList();
     for (String name : classes) {
+      name = name.trim();
       try {
         Class<?> klass = Class.forName(name);
         if (!extClass.isAssignableFrom(klass)) {
@@ -341,4 +275,32 @@ public class RssUtils {
     }
     return serverToPartitions;
   }
+
+  public static void checkProcessedBlockIds(Roaring64NavigableMap blockIdBitmap,
+                                            Roaring64NavigableMap processedBlockIds) {
+    Roaring64NavigableMap cloneBitmap;
+    cloneBitmap = RssUtils.cloneBitMap(blockIdBitmap);
+    cloneBitmap.and(processedBlockIds);
+    if (!blockIdBitmap.equals(cloneBitmap)) {
+      throw new RssException("Blocks read inconsistent: expected " + blockIdBitmap.getLongCardinality()
+          + " blocks, actual " + cloneBitmap.getLongCardinality() + " blocks");
+    }
+  }
+  
+  public static Roaring64NavigableMap generateTaskIdBitMap(Roaring64NavigableMap blockIdBitmap, IdHelper idHelper) {
+    Iterator<Long> iterator = blockIdBitmap.iterator();
+    Roaring64NavigableMap taskIdBitmap = Roaring64NavigableMap.bitmapOf();
+    while (iterator.hasNext()) {
+      taskIdBitmap.addLong(idHelper.getTaskAttemptId(iterator.next()));
+    }
+    return taskIdBitmap;
+  }
+
+  public static List<String> getConfiguredLocalDirs(RssConf conf) {
+    if (conf.getEnv(RSS_LOCAL_DIR_KEY) != null) {
+      return Arrays.asList(conf.getEnv(RSS_LOCAL_DIR_KEY).split(","));
+    } else {
+      return conf.get(RssBaseConf.RSS_STORAGE_BASE_PATH);
+    }
+  }
 }
