@@ -24,11 +24,10 @@ import java.util.Map;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
-import com.google.common.io.Files;
 import org.junit.jupiter.api.AfterEach;
-import org.junit.jupiter.api.BeforeAll;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 import org.roaringbitmap.longlong.Roaring64NavigableMap;
 
 import org.apache.uniffle.client.factory.ShuffleServerClientFactory;
@@ -37,11 +36,12 @@ import org.apache.uniffle.client.impl.ShuffleWriteClientImpl;
 import org.apache.uniffle.client.impl.grpc.ShuffleServerGrpcClient;
 import org.apache.uniffle.client.response.CompressedShuffleBlock;
 import org.apache.uniffle.client.response.SendShuffleDataResult;
-import org.apache.uniffle.client.util.ClientType;
 import org.apache.uniffle.client.util.DefaultIdHelper;
+import org.apache.uniffle.common.ClientType;
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
 import org.apache.uniffle.common.ShuffleBlockInfo;
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleServerInfo;
 import org.apache.uniffle.common.util.RssUtils;
 import org.apache.uniffle.coordinator.CoordinatorConf;
@@ -69,14 +69,12 @@ public class QuorumTest extends ShuffleReadWriteBase {
   private static ShuffleServerInfo fakedShuffleServerInfo2;
   private static ShuffleServerInfo fakedShuffleServerInfo3;
   private static ShuffleServerInfo fakedShuffleServerInfo4;
-  private ShuffleWriteClientImpl shuffleWriteClientImpl;
+  private MockedShuffleWriteClientImpl shuffleWriteClientImpl;
 
-  public static MockedShuffleServer createServer(int id) throws Exception {
+  public static MockedShuffleServer createServer(int id, File tmpDir) throws Exception {
     ShuffleServerConf shuffleServerConf = getShuffleServerConf();
-    shuffleServerConf.setLong("rss.server.app.expired.withoutHeartbeat", 4000);
+    shuffleServerConf.setLong("rss.server.app.expired.withoutHeartbeat", 8000);
     shuffleServerConf.setLong("rss.server.heartbeat.interval", 5000);
-    File tmpDir = Files.createTempDir();
-    tmpDir.deleteOnExit();
     File dataDir1 = new File(tmpDir, id + "_1");
     File dataDir2 = new File(tmpDir, id + "_2");
     String basePath = dataDir1.getAbsolutePath() + "," + dataDir2.getAbsolutePath();
@@ -87,22 +85,19 @@ public class QuorumTest extends ShuffleReadWriteBase {
     return new MockedShuffleServer(shuffleServerConf);
   }
 
-  @BeforeAll
-  public static void initCluster() throws Exception {
+  @BeforeEach
+  public void initCluster(@TempDir File tmpDir) throws Exception {
     CoordinatorConf coordinatorConf = getCoordinatorConf();
     createCoordinatorServer(coordinatorConf);
 
     ShuffleServerConf shuffleServerConf = getShuffleServerConf();
-    shuffleServerConf.setLong("rss.server.app.expired.withoutHeartbeat", 1000);
-    shuffleServerConf.setLong("rss.server.app.expired.withoutHeartbeat", 1000);
-    File tmpDir = Files.createTempDir();
-    tmpDir.deleteOnExit();
+    shuffleServerConf.setLong("rss.server.app.expired.withoutHeartbeat", 8000);
 
-    shuffleServers.add(createServer(0));
-    shuffleServers.add(createServer(1));
-    shuffleServers.add(createServer(2));
-    shuffleServers.add(createServer(3));
-    shuffleServers.add(createServer(4));
+    shuffleServers.add(createServer(0, tmpDir));
+    shuffleServers.add(createServer(1, tmpDir));
+    shuffleServers.add(createServer(2, tmpDir));
+    shuffleServers.add(createServer(3, tmpDir));
+    shuffleServers.add(createServer(4, tmpDir));
 
     shuffleServerInfo0 =
         new ShuffleServerInfo("127.0.0.1-20001", shuffleServers.get(0).getIp(), SHUFFLE_SERVER_PORT + 0);
@@ -132,6 +127,17 @@ public class QuorumTest extends ShuffleReadWriteBase {
         new ShuffleServerInfo("127.0.0.1-20004", shuffleServers.get(2).getIp(), SHUFFLE_SERVER_PORT + 400);
     fakedShuffleServerInfo4 =
         new ShuffleServerInfo("127.0.0.1-20005", shuffleServers.get(2).getIp(), SHUFFLE_SERVER_PORT + 500);
+
+    // spark.rss.data.replica=3
+    // spark.rss.data.replica.write=2
+    // spark.rss.data.replica.read=2
+    ((ShuffleServerGrpcClient)ShuffleServerClientFactory
+        .getInstance().getShuffleServerClient("GRPC", shuffleServerInfo0)).adjustTimeout(200);
+    ((ShuffleServerGrpcClient)ShuffleServerClientFactory
+        .getInstance().getShuffleServerClient("GRPC", shuffleServerInfo1)).adjustTimeout(200);
+    ((ShuffleServerGrpcClient)ShuffleServerClientFactory
+        .getInstance().getShuffleServerClient("GRPC", shuffleServerInfo2)).adjustTimeout(200);
+
     Thread.sleep(2000);
   }
 
@@ -146,26 +152,12 @@ public class QuorumTest extends ShuffleReadWriteBase {
     coordinators = Lists.newArrayList();
   }
 
-  @BeforeEach
-  public void initEnv() throws Exception {
-    // spark.rss.data.replica=3
-    // spark.rss.data.replica.write=2
-    // spark.rss.data.replica.read=2
-    ((ShuffleServerGrpcClient)ShuffleServerClientFactory
-      .getInstance().getShuffleServerClient("GRPC", shuffleServerInfo0)).adjustTimeout(200);
-    ((ShuffleServerGrpcClient)ShuffleServerClientFactory
-      .getInstance().getShuffleServerClient("GRPC", shuffleServerInfo1)).adjustTimeout(200);
-    ((ShuffleServerGrpcClient)ShuffleServerClientFactory
-      .getInstance().getShuffleServerClient("GRPC", shuffleServerInfo2)).adjustTimeout(200);
-  }
-
   @AfterEach
   public void cleanEnv() throws Exception {
     if (shuffleWriteClientImpl != null) {
       shuffleWriteClientImpl.close();
     }
     cleanCluster();
-    initCluster();
     // we need recovery `rpcTime`, or some unit tests may fail
     ((ShuffleServerGrpcClient)ShuffleServerClientFactory
             .getInstance().getShuffleServerClient("GRPC", shuffleServerInfo0)).adjustTimeout(60000);
@@ -262,18 +254,38 @@ public class QuorumTest extends ShuffleReadWriteBase {
         .disableMockedTimeout();
   }
 
+  static class MockedShuffleWriteClientImpl extends ShuffleWriteClientImpl {
+    MockedShuffleWriteClientImpl(String clientType, int retryMax, long retryIntervalMax, int heartBeatThreadNum,
+        int replica, int replicaWrite, int replicaRead, boolean replicaSkipEnabled, int dataTranferPoolSize,
+        int dataCommitPoolSize, int unregisterThreadPoolSize, int unregisterRequestTimeSec) {
+      super(clientType, retryMax, retryIntervalMax, heartBeatThreadNum, replica, replicaWrite, replicaRead,
+          replicaSkipEnabled, dataTranferPoolSize, dataCommitPoolSize, unregisterThreadPoolSize,
+          unregisterRequestTimeSec);
+    }
+
+    public SendShuffleDataResult sendShuffleData(String appId, List<ShuffleBlockInfo> shuffleBlockInfoList) {
+      return super.sendShuffleData(appId, shuffleBlockInfoList, () -> false);
+    }
+  }
+
   private void registerShuffleServer(String testAppId,
       int replica, int replicaWrite, int replicaRead, boolean replicaSkip) {
 
-    shuffleWriteClientImpl = new ShuffleWriteClientImpl(ClientType.GRPC.name(), 3, 1000, 1,
-      replica, replicaWrite, replicaRead, replicaSkip, 1, 1);
+    shuffleWriteClientImpl = new MockedShuffleWriteClientImpl(ClientType.GRPC.name(), 3, 1000, 1,
+      replica, replicaWrite, replicaRead, replicaSkip, 1, 1, 10, 10);
 
     List<ShuffleServerInfo> allServers = Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1,
         shuffleServerInfo2, shuffleServerInfo3, shuffleServerInfo4);
 
     for (int i = 0; i < replica; i++) {
-      shuffleWriteClientImpl.registerShuffle(allServers.get(i),
-          testAppId, 0, Lists.newArrayList(new PartitionRange(0, 0)), new RemoteStorageInfo(""));
+      shuffleWriteClientImpl.registerShuffle(
+          allServers.get(i),
+          testAppId,
+          0,
+          Lists.newArrayList(new PartitionRange(0, 0)),
+          new RemoteStorageInfo(""),
+          ShuffleDataDistributionType.NORMAL
+      );
     }
   }
 
@@ -451,7 +463,7 @@ public class QuorumTest extends ShuffleReadWriteBase {
   }
 
   @Test
-  public void case5() throws Exception {
+  public void case5(@TempDir File tmpDir) throws Exception {
     // this case is to simulate server restarting.
     String testAppId = "case5";
     registerShuffleServer(testAppId, 3, 2, 2, true);
@@ -485,7 +497,7 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     // when one server is restarted, getShuffleResult should success
     shuffleServers.get(1).stopServer();
-    shuffleServers.set(1, createServer(1));
+    shuffleServers.set(1, createServer(1, tmpDir));
     shuffleServers.get(1).start();
     report = shuffleWriteClientImpl.getShuffleResult("GRPC",
       Sets.newHashSet(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2),
@@ -494,7 +506,7 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     // when two servers are restarted, getShuffleResult should fail
     shuffleServers.get(2).stopServer();
-    shuffleServers.set(2, createServer(2));
+    shuffleServers.set(2, createServer(2, tmpDir));
     shuffleServers.get(2).start();
     try {
       report = shuffleWriteClientImpl.getShuffleResult("GRPC",
