@@ -22,17 +22,19 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
-import com.google.common.io.Files;
 import com.google.protobuf.ByteString;
+import org.awaitility.Awaitility;
 import org.junit.jupiter.api.BeforeAll;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 import org.roaringbitmap.longlong.Roaring64NavigableMap;
 
 import org.apache.uniffle.client.api.ShuffleWriteClient;
@@ -47,15 +49,16 @@ import org.apache.uniffle.client.request.RssRegisterShuffleRequest;
 import org.apache.uniffle.client.request.RssReportShuffleResultRequest;
 import org.apache.uniffle.client.request.RssSendCommitRequest;
 import org.apache.uniffle.client.request.RssSendShuffleDataRequest;
-import org.apache.uniffle.client.response.ResponseStatusCode;
 import org.apache.uniffle.client.response.RssGetShuffleResultResponse;
 import org.apache.uniffle.client.response.RssReportShuffleResultResponse;
 import org.apache.uniffle.client.util.ClientUtils;
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
 import org.apache.uniffle.common.ShuffleBlockInfo;
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleServerInfo;
 import org.apache.uniffle.common.config.RssBaseConf;
+import org.apache.uniffle.common.rpc.StatusCode;
 import org.apache.uniffle.common.util.Constants;
 import org.apache.uniffle.coordinator.CoordinatorConf;
 import org.apache.uniffle.proto.RssProtos;
@@ -63,6 +66,7 @@ import org.apache.uniffle.server.ShuffleDataFlushEvent;
 import org.apache.uniffle.server.ShuffleServerConf;
 import org.apache.uniffle.server.ShuffleServerGrpcMetrics;
 import org.apache.uniffle.server.ShuffleServerMetrics;
+import org.apache.uniffle.server.storage.MultiStorageManager;
 import org.apache.uniffle.storage.util.StorageType;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -73,17 +77,16 @@ import static org.junit.jupiter.api.Assertions.fail;
 public class ShuffleServerGrpcTest extends IntegrationTestBase {
 
   private ShuffleServerGrpcClient shuffleServerClient;
-  private AtomicInteger atomicInteger = new AtomicInteger(0);
-  private static Long EVENT_THRESHOLD_SIZE = 2048L;
+  private final AtomicInteger atomicInteger = new AtomicInteger(0);
+  private static final Long EVENT_THRESHOLD_SIZE = 2048L;
   private static final int GB = 1024 * 1024 * 1024;
 
   @BeforeAll
-  public static void setupServers() throws Exception {
+  public static void setupServers(@TempDir File tmpDir) throws Exception {
     CoordinatorConf coordinatorConf = getCoordinatorConf();
     coordinatorConf.setLong(CoordinatorConf.COORDINATOR_APP_EXPIRED, 2000);
     createCoordinatorServer(coordinatorConf);
     ShuffleServerConf shuffleServerConf = getShuffleServerConf();
-    File tmpDir = Files.createTempDir();
     File dataDir1 = new File(tmpDir, "data1");
     String basePath = dataDir1.getAbsolutePath();
     shuffleServerConf.set(ShuffleServerConf.RSS_STORAGE_TYPE, StorageType.MEMORY_LOCALFILE_HDFS.name());
@@ -105,31 +108,35 @@ public class ShuffleServerGrpcTest extends IntegrationTestBase {
   public void clearResourceTest() throws Exception {
     final ShuffleWriteClient shuffleWriteClient =
         ShuffleClientFactory.getInstance().createShuffleWriteClient(
-            "GRPC", 2, 10000L, 4, 1, 1, 1, true, 1, 1);
+            "GRPC", 2, 10000L, 4, 1, 1, 1, true, 1, 1, 10, 10);
     shuffleWriteClient.registerCoordinators("127.0.0.1:19999");
     shuffleWriteClient.registerShuffle(
         new ShuffleServerInfo("127.0.0.1-20001", "127.0.0.1", 20001),
-        "clearResourceTest1",
+        "application_clearResourceTest1",
         0,
-        Lists.newArrayList(new PartitionRange(0, 1)), new RemoteStorageInfo(""));
+        Lists.newArrayList(new PartitionRange(0, 1)),
+        new RemoteStorageInfo(""),
+        ShuffleDataDistributionType.NORMAL
+    );
+    shuffleWriteClient.registerApplicationInfo("application_clearResourceTest1", 500L, "user");
+    shuffleWriteClient.sendAppHeartbeat("application_clearResourceTest1", 500L);
+    shuffleWriteClient.registerApplicationInfo("application_clearResourceTest2", 500L, "user");
+    shuffleWriteClient.sendAppHeartbeat("application_clearResourceTest2", 500L);
 
-    shuffleWriteClient.sendAppHeartbeat("clearResourceTest1", 1000L);
-    shuffleWriteClient.sendAppHeartbeat("clearResourceTest2", 1000L);
-
-    RssRegisterShuffleRequest rrsr = new RssRegisterShuffleRequest("clearResourceTest1", 0,
+    RssRegisterShuffleRequest rrsr = new RssRegisterShuffleRequest("application_clearResourceTest1", 0,
         Lists.newArrayList(new PartitionRange(0, 1)), "");
     shuffleServerClient.registerShuffle(rrsr);
-    rrsr = new RssRegisterShuffleRequest("clearResourceTest2", 0,
+    rrsr = new RssRegisterShuffleRequest("application_clearResourceTest2", 0,
         Lists.newArrayList(new PartitionRange(0, 1)), "");
     shuffleServerClient.registerShuffle(rrsr);
-    assertEquals(Sets.newHashSet("clearResourceTest1", "clearResourceTest2"),
+    assertEquals(Sets.newHashSet("application_clearResourceTest1", "application_clearResourceTest2"),
         shuffleServers.get(0).getShuffleTaskManager().getAppIds());
 
     // Thread will keep refresh clearResourceTest1 in coordinator
     Thread t = new Thread(() -> {
       int i = 0;
       while (i < 20) {
-        shuffleWriteClient.sendAppHeartbeat("clearResourceTest1", 1000L);
+        shuffleWriteClient.sendAppHeartbeat("application_clearResourceTest1", 500L);
         i++;
         try {
           Thread.sleep(1000);
@@ -142,18 +149,19 @@ public class ShuffleServerGrpcTest extends IntegrationTestBase {
 
     // Heartbeat is sent to coordinator too]
     Thread.sleep(3000);
-    shuffleServerClient.registerShuffle(new RssRegisterShuffleRequest("clearResourceTest1", 0,
+    shuffleServerClient.registerShuffle(new RssRegisterShuffleRequest("application_clearResourceTest1", 0,
         Lists.newArrayList(new PartitionRange(0, 1)), ""));
-    assertEquals(Sets.newHashSet("clearResourceTest1"),
+    assertEquals(Sets.newHashSet("application_clearResourceTest1"),
         coordinators.get(0).getApplicationManager().getAppIds());
     // clearResourceTest2 will be removed because of rss.server.app.expired.withoutHeartbeat
     Thread.sleep(2000);
-    assertEquals(Sets.newHashSet("clearResourceTest1"),
+    assertEquals(Sets.newHashSet("application_clearResourceTest1"),
         shuffleServers.get(0).getShuffleTaskManager().getAppIds());
 
     // clearResourceTest1 will be removed because of rss.server.app.expired.withoutHeartbeat
     t.interrupt();
-    Thread.sleep(8000);
+    Awaitility.await().timeout(20, TimeUnit.SECONDS).until(
+        () -> shuffleServers.get(0).getShuffleTaskManager().getAppIds().size() == 0);
     assertEquals(0, shuffleServers.get(0).getShuffleTaskManager().getAppIds().size());
 
   }
@@ -197,7 +205,7 @@ public class ShuffleServerGrpcTest extends IntegrationTestBase {
     request =
         new RssReportShuffleResultRequest("shuffleResultTest", 0, 0L, partitionToBlockIds, 1);
     RssReportShuffleResultResponse response = shuffleServerClient.reportShuffleResult(request);
-    assertEquals(ResponseStatusCode.SUCCESS, response.getStatusCode());
+    assertEquals(StatusCode.SUCCESS, response.getStatusCode());
     req = new RssGetShuffleResultRequest("shuffleResultTest", 0, 1);
     result = shuffleServerClient.getShuffleResult(req);
     blockIdBitmap = result.getBlockIdBitmap();
@@ -369,7 +377,8 @@ public class ShuffleServerGrpcTest extends IntegrationTestBase {
         EVENT_THRESHOLD_SIZE + 1, null, null, null);
     try {
       // can't find storage info with appId2
-      shuffleServers.get(0).getStorageManager().selectStorage(event2).getStoragePath();
+      ((MultiStorageManager)shuffleServers.get(0).getStorageManager()).getColdStorageManager()
+          .selectStorage(event2).getStoragePath();
       fail("Exception should be thrown with un-register appId");
     } catch (Exception e) {
       // expected exception, ignore
@@ -392,8 +401,6 @@ public class ShuffleServerGrpcTest extends IntegrationTestBase {
     RssSendShuffleDataRequest rssdr = new RssSendShuffleDataRequest(
         "sendDataWithoutRegisterTest", 3, 1000, shuffleToBlocks);
     shuffleServerClient.sendShuffleData(rssdr);
-    assertEquals(132, shuffleServers.get(0).getPreAllocatedMemory());
-    Thread.sleep(10000);
     assertEquals(0, shuffleServers.get(0).getPreAllocatedMemory());
   }
 
@@ -508,7 +515,7 @@ public class ShuffleServerGrpcTest extends IntegrationTestBase {
 
   @Disabled("flaky test")
   @Test
-  public void rpcMetricsTest() {
+  public void rpcMetricsTest() throws Exception {
     String appId = "rpcMetricsTest";
     int shuffleId = 0;
     final double oldGrpcTotal = shuffleServers.get(0).getGrpcMetrics().getCounterGrpcTotal().get();
