@@ -19,10 +19,7 @@ package org.apache.uniffle.server;
 
 public abstract class Checker {
 
-  protected ShuffleServerConf conf;
-
   Checker(ShuffleServerConf conf) {
-    this.conf = conf;
   }
 
   abstract boolean checkIsHealthy();
