@@ -18,9 +18,12 @@
 package org.apache.uniffle.server;
 
 import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.Executors;
@@ -33,7 +36,9 @@ import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Queues;
+import com.google.common.collect.Range;
 import com.google.common.collect.Sets;
+import org.apache.commons.collections.CollectionUtils;
 import org.roaringbitmap.longlong.LongIterator;
 import org.roaringbitmap.longlong.Roaring64NavigableMap;
 import org.slf4j.Logger;
@@ -41,20 +46,28 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleDataResult;
 import org.apache.uniffle.common.ShuffleIndexResult;
 import org.apache.uniffle.common.ShufflePartitionedBlock;
 import org.apache.uniffle.common.ShufflePartitionedData;
 import org.apache.uniffle.common.config.RssBaseConf;
+import org.apache.uniffle.common.exception.FileNotFoundException;
+import org.apache.uniffle.common.rpc.StatusCode;
 import org.apache.uniffle.common.util.Constants;
 import org.apache.uniffle.common.util.RssUtils;
 import org.apache.uniffle.common.util.ThreadUtils;
 import org.apache.uniffle.server.buffer.PreAllocatedBufferInfo;
+import org.apache.uniffle.server.buffer.ShuffleBuffer;
 import org.apache.uniffle.server.buffer.ShuffleBufferManager;
+import org.apache.uniffle.server.event.AppPurgeEvent;
+import org.apache.uniffle.server.event.PurgeEvent;
+import org.apache.uniffle.server.event.ShufflePurgeEvent;
 import org.apache.uniffle.server.storage.StorageManager;
 import org.apache.uniffle.storage.common.Storage;
 import org.apache.uniffle.storage.common.StorageReadMetrics;
 import org.apache.uniffle.storage.request.CreateShuffleReadHandlerRequest;
+import org.apache.uniffle.storage.util.ShuffleStorageUtils;
 
 public class ShuffleTaskManager {
 
@@ -62,24 +75,27 @@ public class ShuffleTaskManager {
   private final ShuffleFlushManager shuffleFlushManager;
   private final ScheduledExecutorService scheduledExecutorService;
   private final ScheduledExecutorService expiredAppCleanupExecutorService;
+  private final ScheduledExecutorService leakShuffleDataCheckExecutorService;
+  private ScheduledExecutorService triggerFlushExecutorService;
   private final StorageManager storageManager;
   private AtomicLong requireBufferId = new AtomicLong(0);
   private ShuffleServerConf conf;
   private long appExpiredWithoutHB;
   private long preAllocationExpired;
   private long commitCheckIntervalMax;
+  private long leakShuffleDataCheckInterval;
+  private long triggerFlushInterval;
   // appId -> shuffleId -> blockIds to avoid too many appId
   // store taskAttemptId info to filter speculation task
   // Roaring64NavigableMap instance will cost much memory,
   // merge different blockId of partition to one bitmap can reduce memory cost,
   // but when get blockId, performance will degrade a little which can be optimized by client configuration
   private Map<String, Map<Integer, Roaring64NavigableMap[]>> partitionsToBlockIds;
-  private ShuffleBufferManager shuffleBufferManager;
+  private final ShuffleBufferManager shuffleBufferManager;
   private Map<String, ShuffleTaskInfo> shuffleTaskInfos = Maps.newConcurrentMap();
   private Map<Long, PreAllocatedBufferInfo> requireBufferIds = Maps.newConcurrentMap();
   private Runnable clearResourceThread;
-  private BlockingQueue<String> expiredAppIdQueue = Queues.newLinkedBlockingQueue();
-  // appId -> shuffleId -> serverReadHandler
+  private BlockingQueue<PurgeEvent> expiredAppIdQueue = Queues.newLinkedBlockingQueue();
 
   public ShuffleTaskManager(
       ShuffleServerConf conf,
@@ -94,23 +110,42 @@ public class ShuffleTaskManager {
     this.appExpiredWithoutHB = conf.getLong(ShuffleServerConf.SERVER_APP_EXPIRED_WITHOUT_HEARTBEAT);
     this.commitCheckIntervalMax = conf.getLong(ShuffleServerConf.SERVER_COMMIT_CHECK_INTERVAL_MAX);
     this.preAllocationExpired = conf.getLong(ShuffleServerConf.SERVER_PRE_ALLOCATION_EXPIRED);
+    this.leakShuffleDataCheckInterval = conf.getLong(ShuffleServerConf.SERVER_LEAK_SHUFFLE_DATA_CHECK_INTERVAL);
+    this.triggerFlushInterval = conf.getLong(ShuffleServerConf.SERVER_TRIGGER_FLUSH_CHECK_INTERVAL);
     // the thread for checking application status
     this.scheduledExecutorService = Executors.newSingleThreadScheduledExecutor(
         ThreadUtils.getThreadFactory("checkResource-%d"));
     scheduledExecutorService.scheduleAtFixedRate(
-        () -> preAllocatedBufferCheck(), preAllocationExpired / 2,
+        this::preAllocatedBufferCheck, preAllocationExpired / 2,
         preAllocationExpired / 2, TimeUnit.MILLISECONDS);
     this.expiredAppCleanupExecutorService = Executors.newSingleThreadScheduledExecutor(
         ThreadUtils.getThreadFactory("expiredAppCleaner"));
     expiredAppCleanupExecutorService.scheduleAtFixedRate(
-        () -> checkResourceStatus(), appExpiredWithoutHB / 2,
+        this::checkResourceStatus, appExpiredWithoutHB / 2,
         appExpiredWithoutHB / 2, TimeUnit.MILLISECONDS);
+    this.leakShuffleDataCheckExecutorService = Executors.newSingleThreadScheduledExecutor(
+        ThreadUtils.getThreadFactory("leakShuffleDataChecker"));
+    leakShuffleDataCheckExecutorService.scheduleAtFixedRate(
+        this::checkLeakShuffleData, leakShuffleDataCheckInterval,
+            leakShuffleDataCheckInterval, TimeUnit.MILLISECONDS);
+    if (triggerFlushInterval > 0) {
+      triggerFlushExecutorService = Executors.newSingleThreadScheduledExecutor(
+          ThreadUtils.getThreadFactory("triggerShuffleBufferManagerFlush"));
+      triggerFlushExecutorService.scheduleWithFixedDelay(
+              this::triggerFlush, triggerFlushInterval / 2,
+          triggerFlushInterval, TimeUnit.MILLISECONDS);
+    }
     // the thread for clear expired resources
     clearResourceThread = () -> {
       while (true) {
         try {
-          String appId = expiredAppIdQueue.take();
-          removeResources(appId);
+          PurgeEvent event = expiredAppIdQueue.take();
+          if (event instanceof AppPurgeEvent) {
+            removeResources(event.getAppId());
+          }
+          if (event instanceof ShufflePurgeEvent) {
+            removeResourcesByShuffleIds(event.getAppId(), event.getShuffleIds());
+          }
         } catch (Exception e) {
           LOG.error("Exception happened when clear resource for expired application", e);
         }
@@ -122,14 +157,36 @@ public class ShuffleTaskManager {
     thread.start();
   }
 
+  /**
+   * Only for test
+   */
+  @VisibleForTesting
   public StatusCode registerShuffle(
       String appId,
       int shuffleId,
       List<PartitionRange> partitionRanges,
       RemoteStorageInfo remoteStorageInfo,
       String user) {
+    return registerShuffle(
+        appId,
+        shuffleId,
+        partitionRanges,
+        remoteStorageInfo,
+        user,
+        ShuffleDataDistributionType.NORMAL
+    );
+  }
+
+  public StatusCode registerShuffle(
+      String appId,
+      int shuffleId,
+      List<PartitionRange> partitionRanges,
+      RemoteStorageInfo remoteStorageInfo,
+      String user,
+      ShuffleDataDistributionType dataDistType) {
     refreshAppId(appId);
     shuffleTaskInfos.get(appId).setUser(user);
+    shuffleTaskInfos.get(appId).setDataDistType(dataDistType);
     partitionsToBlockIds.putIfAbsent(appId, Maps.newConcurrentMap());
     for (PartitionRange partitionRange : partitionRanges) {
       shuffleBufferManager.registerBuffer(appId, shuffleId, partitionRange.getStart(), partitionRange.getEnd());
@@ -143,15 +200,29 @@ public class ShuffleTaskManager {
   public StatusCode cacheShuffleData(
       String appId, int shuffleId, boolean isPreAllocated, ShufflePartitionedData spd) {
     refreshAppId(appId);
-    return shuffleBufferManager.cacheShuffleData(appId, shuffleId, isPreAllocated, spd);
+    return shuffleBufferManager.cacheShuffleData(
+        appId,
+        shuffleId,
+        isPreAllocated,
+        spd,
+        this::getPartitionDataSize
+    );
   }
 
-  public boolean isPreAllocated(long requireBufferId) {
-    return requireBufferIds.containsKey(requireBufferId);
+  public PreAllocatedBufferInfo getAndRemovePreAllocatedBuffer(long requireBufferId) {
+    return requireBufferIds.remove(requireBufferId);
   }
 
-  public void removeRequireBufferId(long requireId) {
-    requireBufferIds.remove(requireId);
+  public void releasePreAllocatedSize(long requireSize) {
+    shuffleBufferManager.releasePreAllocatedSize(requireSize);
+  }
+
+  @VisibleForTesting
+  void removeAndReleasePreAllocatedBuffer(long requireBufferId) {
+    PreAllocatedBufferInfo info = getAndRemovePreAllocatedBuffer(requireBufferId);
+    if (info != null) {
+      releasePreAllocatedSize(info.getRequireSize());
+    }
   }
 
   public StatusCode commitShuffle(String appId, int shuffleId) throws Exception {
@@ -159,7 +230,7 @@ public class ShuffleTaskManager {
     refreshAppId(appId);
     Roaring64NavigableMap cachedBlockIds = getCachedBlockIds(appId, shuffleId);
     Roaring64NavigableMap cloneBlockIds;
-    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo());
+    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo(appId));
     Object lock = shuffleTaskInfo.getCommitLocks().computeIfAbsent(shuffleId, x -> new Object());
     synchronized (lock) {
       long commitTimeout = conf.get(ShuffleServerConf.SERVER_COMMIT_TIMEOUT);
@@ -203,6 +274,9 @@ public class ShuffleTaskManager {
       String appId, Integer shuffleId, Map<Integer, long[]> partitionToBlockIds, int bitmapNum) {
     refreshAppId(appId);
     Map<Integer, Roaring64NavigableMap[]> shuffleIdToPartitions = partitionsToBlockIds.get(appId);
+    if (shuffleIdToPartitions == null) {
+      throw new RuntimeException("appId[" + appId  + "] is expired!");
+    }
     if (!shuffleIdToPartitions.containsKey(shuffleId)) {
       Roaring64NavigableMap[] blockIds = new Roaring64NavigableMap[bitmapNum];
       for (int i = 0; i < bitmapNum; i++) {
@@ -223,29 +297,48 @@ public class ShuffleTaskManager {
   }
 
   public int updateAndGetCommitCount(String appId, int shuffleId) {
-    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo());
+    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo(appId));
     AtomicInteger commitNum = shuffleTaskInfo.getCommitCounts()
         .computeIfAbsent(shuffleId, x -> new AtomicInteger(0));
     return commitNum.incrementAndGet();
   }
 
+  // Only for tests
   public void updateCachedBlockIds(String appId, int shuffleId, ShufflePartitionedBlock[] spbs) {
+    updateCachedBlockIds(appId, shuffleId, 0, spbs);
+  }
+
+  public void updateCachedBlockIds(String appId, int shuffleId, int partitionId, ShufflePartitionedBlock[] spbs) {
     if (spbs == null || spbs.length == 0) {
       return;
     }
-    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo());
+    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo(appId));
     Roaring64NavigableMap bitmap = shuffleTaskInfo.getCachedBlockIds()
         .computeIfAbsent(shuffleId, x -> Roaring64NavigableMap.bitmapOf());
+
+    long size = 0L;
     synchronized (bitmap) {
       for (ShufflePartitionedBlock spb : spbs) {
         bitmap.addLong(spb.getBlockId());
+        size += spb.getSize();
       }
     }
+    long partitionSize = shuffleTaskInfo.addPartitionDataSize(
+        shuffleId,
+        partitionId,
+        size
+    );
+    if (shuffleBufferManager.isHugePartition(partitionSize)) {
+      shuffleTaskInfo.markHugePartition(
+          shuffleId,
+          partitionId
+      );
+    }
   }
 
   public Roaring64NavigableMap getCachedBlockIds(String appId, int shuffleId) {
     Map<Integer, Roaring64NavigableMap> shuffleIdToBlockIds = shuffleTaskInfos
-        .getOrDefault(appId, new ShuffleTaskInfo()).getCachedBlockIds();
+        .getOrDefault(appId, new ShuffleTaskInfo(appId)).getCachedBlockIds();
     Roaring64NavigableMap blockIds = shuffleIdToBlockIds.get(shuffleId);
     if (blockIds == null) {
       LOG.warn("Unexpected value when getCachedBlockIds for appId[" + appId + "], shuffleId[" + shuffleId + "]");
@@ -254,6 +347,28 @@ public class ShuffleTaskManager {
     return blockIds;
   }
 
+  public long getPartitionDataSize(String appId, int shuffleId, int partitionId) {
+    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.get(appId);
+    if (shuffleTaskInfo == null) {
+      return 0L;
+    }
+    return shuffleTaskInfo.getPartitionDataSize(shuffleId, partitionId);
+  }
+
+  public long requireBuffer(String appId, int shuffleId, List<Integer> partitionIds, int requireSize) {
+    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.get(appId);
+    if (shuffleTaskInfo != null) {
+      for (int partitionId : partitionIds) {
+        long partitionUsedDataSize = getPartitionDataSize(appId, shuffleId, partitionId);
+        if (shuffleBufferManager.limitHugePartition(appId, shuffleId, partitionId, partitionUsedDataSize)) {
+          ShuffleServerMetrics.counterTotalRequireBufferFailedForHugePartition.inc();
+          return -1;
+        }
+      }
+    }
+    return requireBuffer(requireSize);
+  }
+
   public long requireBuffer(int requireSize) {
     long requireId = -1;
     if (shuffleBufferManager.requireMemory(requireSize, true)) {
@@ -261,15 +376,34 @@ public class ShuffleTaskManager {
       requireBufferIds.put(requireId,
           new PreAllocatedBufferInfo(requireId, System.currentTimeMillis(), requireSize));
     }
+    if (requireId == -1) {
+      ShuffleServerMetrics.counterTotalRequireBufferFailedForRegularPartition.inc();
+    }
     return requireId;
   }
 
   public byte[] getFinishedBlockIds(String appId, Integer shuffleId, Set<Integer> partitions) throws IOException {
     refreshAppId(appId);
     for (int partitionId : partitions) {
-      Storage storage = storageManager.selectStorage(new ShuffleDataReadEvent(appId, shuffleId, partitionId));
+      Map.Entry<Range<Integer>, ShuffleBuffer> entry =
+          shuffleBufferManager.getShuffleBufferEntry(appId, shuffleId, partitionId);
+      if (entry == null) {
+        LOG.error("The empty shuffle buffer, this should not happen. appId: {}, shuffleId: {}, partition: {}",
+            appId, shuffleId, partitionId);
+        continue;
+      }
+      Storage storage = storageManager.selectStorage(
+          new ShuffleDataReadEvent(
+              appId,
+              shuffleId,
+              partitionId,
+              entry.getKey().lowerEndpoint()
+          )
+      );
       // update shuffle's timestamp that was recently read.
-      storage.updateReadMetrics(new StorageReadMetrics(appId, shuffleId));
+      if (storage != null) {
+        storage.updateReadMetrics(new StorageReadMetrics(appId, shuffleId));
+      }
     }
     Map<Integer, Roaring64NavigableMap[]> shuffleIdToPartitions = partitionsToBlockIds.get(appId);
     if (shuffleIdToPartitions == null) {
@@ -317,9 +451,10 @@ public class ShuffleTaskManager {
   }
 
   public ShuffleDataResult getInMemoryShuffleData(
-      String appId, Integer shuffleId, Integer partitionId, long blockId, int readBufferSize) {
+      String appId, Integer shuffleId, Integer partitionId, long blockId, int readBufferSize,
+      Roaring64NavigableMap expectedTaskIds) {
     return shuffleBufferManager.getShuffleData(appId,
-        shuffleId, partitionId, blockId, readBufferSize);
+        shuffleId, partitionId, blockId, readBufferSize, expectedTaskIds);
   }
 
   public ShuffleDataResult getShuffleData(
@@ -335,7 +470,11 @@ public class ShuffleTaskManager {
     request.setPartitionNum(partitionNum);
     request.setStorageType(storageType);
     request.setRssBaseConf(conf);
-    Storage storage = storageManager.selectStorage(new ShuffleDataReadEvent(appId, shuffleId, partitionId));
+    int[] range = ShuffleStorageUtils.getPartitionRange(partitionId, partitionNumPerRange, partitionNum);
+    Storage storage = storageManager.selectStorage(new ShuffleDataReadEvent(appId, shuffleId, partitionId, range[0]));
+    if (storage == null) {
+      throw new FileNotFoundException("No such data stored in current storage manager.");
+    }
 
     return storage.getOrCreateReadHandler(request).getShuffleData(offset, length);
   }
@@ -356,8 +495,11 @@ public class ShuffleTaskManager {
     request.setPartitionNum(partitionNum);
     request.setStorageType(storageType);
     request.setRssBaseConf(conf);
-
-    Storage storage = storageManager.selectStorage(new ShuffleDataReadEvent(appId, shuffleId, partitionId));
+    int[] range = ShuffleStorageUtils.getPartitionRange(partitionId, partitionNumPerRange, partitionNum);
+    Storage storage = storageManager.selectStorage(new ShuffleDataReadEvent(appId, shuffleId, partitionId, range[0]));
+    if (storage == null) {
+      throw new FileNotFoundException("No such data in current storage manager.");
+    }
     return storage.getOrCreateReadHandler(request).getShuffleIndex();
   }
 
@@ -369,7 +511,7 @@ public class ShuffleTaskManager {
         if (System.currentTimeMillis() - shuffleTaskInfos.get(appId).getCurrentTimes() > appExpiredWithoutHB) {
           LOG.info("Detect expired appId[" + appId + "] according "
               + "to rss.server.app.expired.withoutHeartbeat");
-          expiredAppIdQueue.add(appId);
+          expiredAppIdQueue.add(new AppPurgeEvent(appId, getUserByAppId(appId)));
         }
       }
       ShuffleServerMetrics.gaugeAppNum.set(shuffleTaskInfos.size());
@@ -378,23 +520,83 @@ public class ShuffleTaskManager {
     }
   }
 
+  /**
+   * Clear up the partial resources of shuffleIds of App.
+   * @param appId
+   * @param shuffleIds
+   */
+  public void removeResourcesByShuffleIds(String appId, List<Integer> shuffleIds) {
+    if (CollectionUtils.isEmpty(shuffleIds)) {
+      return;
+    }
+
+    LOG.info("Start remove resource for appId[{}], shuffleIds[{}]", appId, shuffleIds);
+    final long start = System.currentTimeMillis();
+    final ShuffleTaskInfo taskInfo = shuffleTaskInfos.get(appId);
+    if (taskInfo != null) {
+      for (Integer shuffleId : shuffleIds) {
+        taskInfo.getCachedBlockIds().remove(shuffleId);
+        taskInfo.getCommitCounts().remove(shuffleId);
+        taskInfo.getCommitLocks().remove(shuffleId);
+      }
+    }
+    Optional.ofNullable(partitionsToBlockIds.get(appId)).ifPresent(x -> {
+      for (Integer shuffleId : shuffleIds) {
+        x.remove(shuffleId);
+      }
+    });
+    shuffleBufferManager.removeBufferByShuffleId(appId, shuffleIds);
+    shuffleFlushManager.removeResourcesOfShuffleId(appId, shuffleIds);
+    storageManager.removeResources(
+        new ShufflePurgeEvent(appId, getUserByAppId(appId), shuffleIds)
+    );
+    LOG.info("Finish remove resource for appId[{}], shuffleIds[{}], cost[{}]",
+        appId, shuffleIds, System.currentTimeMillis() - start);
+  }
+
+  public void checkLeakShuffleData() {
+    LOG.info("Start check leak shuffle data");
+    try {
+      Set<String> appIds = Sets.newHashSet(shuffleTaskInfos.keySet());
+      storageManager.checkAndClearLeakedShuffleData(appIds);
+      LOG.info("Finish check leak shuffle data");
+    } catch (Exception e) {
+      LOG.warn("Error happened in checkLeakShuffleData", e);
+    }
+  }
+
   @VisibleForTesting
   public void removeResources(String appId) {
     LOG.info("Start remove resource for appId[" + appId + "]");
     final long start = System.currentTimeMillis();
-    final Map<Integer, Roaring64NavigableMap> shuffleToCachedBlockIds = shuffleTaskInfos.get(appId).getCachedBlockIds();
+    ShuffleTaskInfo shuffleTaskInfo = shuffleTaskInfos.remove(appId);
+    if (shuffleTaskInfo == null) {
+      LOG.info("Resource for appId[" + appId + "] had been removed before.");
+      return;
+    }
+    final Map<Integer, Roaring64NavigableMap> shuffleToCachedBlockIds = shuffleTaskInfo.getCachedBlockIds();
     partitionsToBlockIds.remove(appId);
     shuffleBufferManager.removeBuffer(appId);
     shuffleFlushManager.removeResources(appId);
     if (!shuffleToCachedBlockIds.isEmpty()) {
-      storageManager.removeResources(appId, shuffleToCachedBlockIds.keySet(), getUserByAppId(appId));
+      storageManager.removeResources(
+          new AppPurgeEvent(appId, getUserByAppId(appId), new ArrayList<>(shuffleToCachedBlockIds.keySet()))
+      );
+    }
+    if (shuffleTaskInfo.hasHugePartition()) {
+      ShuffleServerMetrics.gaugeAppWithHugePartitionNum.dec();
+      ShuffleServerMetrics.gaugeHugePartitionNum.dec(shuffleTaskInfo.getHugePartitionSize());
     }
-    shuffleTaskInfos.remove(appId);
     LOG.info("Finish remove resource for appId[" + appId + "] cost " + (System.currentTimeMillis() - start) + " ms");
   }
 
   public void refreshAppId(String appId) {
-    shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo()).setCurrentTimes(System.currentTimeMillis());
+    shuffleTaskInfos.computeIfAbsent(
+        appId,
+        x -> {
+          ShuffleServerMetrics.counterTotalAppNum.inc();
+          return new ShuffleTaskInfo(appId);
+        }).setCurrentTimes(System.currentTimeMillis());
   }
 
   // check pre allocated buffer, release the memory if it expired
@@ -405,12 +607,17 @@ public class ShuffleTaskManager {
       for (PreAllocatedBufferInfo info : requireBufferIds.values()) {
         if (current - info.getTimestamp() > preAllocationExpired) {
           removeIds.add(info.getRequireId());
-          shuffleBufferManager.releaseMemory(info.getRequireSize(), false, true);
         }
       }
       for (Long requireId : removeIds) {
-        requireBufferIds.remove(requireId);
-        LOG.info("Remove expired requireId " + requireId);
+        PreAllocatedBufferInfo info = requireBufferIds.remove(requireId);
+        if (info != null) {
+          // move release memory code down to here as the requiredBuffer could be consumed during removing processing.
+          shuffleBufferManager.releaseMemory(info.getRequireSize(), false, true);
+          LOG.info("Remove expired preAllocatedBuffer " + requireId);
+        } else {
+          LOG.info("PreAllocatedBuffer[id={}] has already been removed", requireId);
+        }
       }
     } catch (Exception e) {
       LOG.warn("Error happened in preAllocatedBufferCheck", e);
@@ -426,7 +633,7 @@ public class ShuffleTaskManager {
   }
 
   public String getUserByAppId(String appId) {
-    return shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo()).getUser();
+    return shuffleTaskInfos.computeIfAbsent(appId, x -> new ShuffleTaskInfo(appId)).getUser();
   }
 
   @VisibleForTesting
@@ -443,4 +650,28 @@ public class ShuffleTaskManager {
   public Map<String, Map<Integer, Roaring64NavigableMap[]>> getPartitionsToBlockIds() {
     return partitionsToBlockIds;
   }
+
+  public void removeShuffleDataAsync(String appId, int shuffleId) {
+    expiredAppIdQueue.add(new ShufflePurgeEvent(appId, getUserByAppId(appId), Arrays.asList(shuffleId)));
+  }
+
+  @VisibleForTesting
+  void removeShuffleDataSync(String appId, int shuffleId) {
+    removeResourcesByShuffleIds(appId, Arrays.asList(shuffleId));
+  }
+
+  public ShuffleDataDistributionType getDataDistributionType(String appId) {
+    return shuffleTaskInfos.get(appId).getDataDistType();
+  }
+
+  @VisibleForTesting
+  public ShuffleTaskInfo getShuffleTaskInfo(String appId) {
+    return shuffleTaskInfos.get(appId);
+  }
+
+  private void triggerFlush() {
+    synchronized (this.shuffleBufferManager) {
+      this.shuffleBufferManager.flushIfNecessary();
+    }
+  }
 }
