@@ -27,6 +27,7 @@ import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.function.Supplier;
+import java.util.stream.IntStream;
 
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
@@ -52,12 +53,16 @@ import org.apache.uniffle.common.ShufflePartitionedBlock;
 import org.apache.uniffle.common.config.RssBaseConf;
 import org.apache.uniffle.common.util.ChecksumUtils;
 import org.apache.uniffle.server.buffer.ShuffleBufferManager;
+import org.apache.uniffle.server.event.AppPurgeEvent;
 import org.apache.uniffle.server.storage.HdfsStorageManager;
+import org.apache.uniffle.server.storage.LocalStorageManagerFallbackStrategy;
+import org.apache.uniffle.server.storage.MultiStorageManager;
 import org.apache.uniffle.server.storage.StorageManager;
 import org.apache.uniffle.server.storage.StorageManagerFactory;
 import org.apache.uniffle.storage.HdfsTestBase;
 import org.apache.uniffle.storage.common.AbstractStorage;
 import org.apache.uniffle.storage.common.HdfsStorage;
+import org.apache.uniffle.storage.common.LocalStorage;
 import org.apache.uniffle.storage.handler.impl.HdfsClientReadHandler;
 import org.apache.uniffle.storage.util.StorageType;
 
@@ -113,6 +118,35 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
     assertEquals("value", manager.getHadoopConf().get("a.b"));
   }
 
+  @Test
+  public void concurrentWrite2HdfsWriteOfSinglePartition() throws Exception {
+    ShuffleServerConf shuffleServerConf = new ShuffleServerConf();
+    shuffleServerConf.set(ShuffleServerConf.RSS_STORAGE_BASE_PATH, Collections.emptyList());
+    shuffleServerConf.setString(ShuffleServerConf.RSS_STORAGE_TYPE, StorageType.HDFS.name());
+    int maxConcurrency = 3;
+    shuffleServerConf.setInteger(ShuffleServerConf.SERVER_MAX_CONCURRENCY_OF_ONE_PARTITION, maxConcurrency);
+
+    String appId = "concurrentWrite2HdfsWriteOfSinglePartition_appId";
+    StorageManager storageManager =
+        StorageManagerFactory.getInstance().createStorageManager(shuffleServerConf);
+    storageManager.registerRemoteStorage(appId, remoteStorage);
+    ShuffleFlushManager manager =
+        new ShuffleFlushManager(shuffleServerConf, "shuffleServerId", mockShuffleServer, storageManager);
+
+    IntStream.range(0, 20).forEach(x -> {
+      ShuffleDataFlushEvent event = createShuffleDataFlushEvent(appId, 1, 1, 1, null);
+      manager.addToFlushQueue(event);
+    });
+    waitForFlush(manager, appId, 1, 10 * 5);
+
+    FileStatus[] fileStatuses = fs.listStatus(new Path(HDFS_URI + "/rss/test/" + appId + "/1/1-1"));
+    long actual = Arrays.stream(fileStatuses).filter(x -> x.getPath().getName().endsWith("data")).count();
+
+    assertEquals(maxConcurrency, actual);
+    actual = Arrays.stream(fileStatuses).filter(x -> x.getPath().getName().endsWith("index")).count();
+    assertEquals(maxConcurrency, actual);
+  }
+
   @Test
   public void writeTest() throws Exception {
     String appId = "writeTest_appId";
@@ -229,7 +263,9 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
     manager.removeResources(appId1);
 
     assertTrue(((HdfsStorageManager)storageManager).getAppIdToStorages().containsKey(appId1));
-    storageManager.removeResources(appId1, Sets.newHashSet(1), StringUtils.EMPTY);
+    storageManager.removeResources(
+        new AppPurgeEvent(appId1, StringUtils.EMPTY, Arrays.asList(1))
+    );
     assertFalse(((HdfsStorageManager)storageManager).getAppIdToStorages().containsKey(appId1));
     try {
       fs.listStatus(new Path(remoteStorage.getPath() + "/" + appId1 + "/"));
@@ -244,16 +280,20 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
     assertEquals(1, size);
     manager.removeResources(appId2);
     assertTrue(((HdfsStorageManager)storageManager).getAppIdToStorages().containsKey(appId2));
-    storageManager.removeResources(appId2, Sets.newHashSet(1), StringUtils.EMPTY);
+    storageManager.removeResources(
+        new AppPurgeEvent(appId2, StringUtils.EMPTY, Arrays.asList(1))
+    );
     assertFalse(((HdfsStorageManager)storageManager).getAppIdToStorages().containsKey(appId2));
     assertEquals(0, manager.getCommittedBlockIds(appId2, 1).getLongCardinality());
     size = storage.getHandlerSize();
     assertEquals(0, size);
     // fs create a remoteStorage for appId2 before remove resources,
-    // but thecache from appIdToStorages has removed, so we need to delete this path in hdfs
+    // but the cache from appIdToStorages has been removed, so we need to delete this path in hdfs
     Path path = new Path(remoteStorage.getPath() + "/" + appId2 + "/");
     assertTrue(fs.mkdirs(path));
-    storageManager.removeResources(appId2, Sets.newHashSet(1), StringUtils.EMPTY);
+    storageManager.removeResources(
+        new AppPurgeEvent(appId2, StringUtils.EMPTY, Lists.newArrayList(1))
+    );
     assertFalse(fs.exists(path));
     HdfsStorage storageByAppId = ((HdfsStorageManager) storageManager).getStorageByAppId(appId2);
     assertNull(storageByAppId);
@@ -286,7 +326,9 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
     assertEquals(2, storage.getHandlerSize());
     File file = new File(tempDir, appId1);
     assertTrue(file.exists());
-    storageManager.removeResources(appId1, Sets.newHashSet(1), StringUtils.EMPTY);
+    storageManager.removeResources(
+        new AppPurgeEvent(appId1, StringUtils.EMPTY, Lists.newArrayList(1))
+    );
     manager.removeResources(appId1);
     assertFalse(file.exists());
     ShuffleDataFlushEvent event3 =
@@ -297,7 +339,9 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
     assertEquals(5, manager.getCommittedBlockIds(appId2, 1).getLongCardinality());
     assertEquals(1, storage.getHandlerSize());
     manager.removeResources(appId2);
-    storageManager.removeResources(appId2, Sets.newHashSet(1), StringUtils.EMPTY);
+    storageManager.removeResources(
+        new AppPurgeEvent(appId2, StringUtils.EMPTY, Lists.newArrayList(1))
+    );
     assertEquals(0, manager.getCommittedBlockIds(appId2, 1).getLongCardinality());
     assertEquals(0, storage.getHandlerSize());
   }
@@ -349,9 +393,26 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
 
   public static ShuffleDataFlushEvent createShuffleDataFlushEvent(
       String appId, int shuffleId, int startPartition, int endPartition, Supplier<Boolean> isValid) {
+    return createShuffleDataFlushEvent(
+      appId,
+      shuffleId,
+        startPartition,
+        endPartition,
+        isValid,
+        1
+    );
+  }
+
+  public static ShuffleDataFlushEvent createShuffleDataFlushEvent(
+      String appId,
+      int shuffleId,
+      int startPartition,
+      int endPartition,
+      Supplier<Boolean> isValid,
+      int size) {
     List<ShufflePartitionedBlock> spbs = createBlock(5, 32);
     return new ShuffleDataFlushEvent(ATOMIC_LONG.getAndIncrement(),
-        appId, shuffleId, startPartition, endPartition, 1, spbs, isValid, null);
+        appId, shuffleId, startPartition, endPartition, size, spbs, isValid, null);
   }
 
   public static List<ShufflePartitionedBlock> createBlock(int num, int length) {
@@ -405,6 +466,55 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
     assertEquals(blocks.size(), matchNum);
   }
 
+  @Test
+  public void fallbackWrittenWhenMultiStorageManagerEnableTest(@TempDir File tempDir) throws InterruptedException {
+    shuffleServerConf.setLong(ShuffleServerConf.FLUSH_COLD_STORAGE_THRESHOLD_SIZE, 10000L);
+    shuffleServerConf.set(RssBaseConf.RSS_STORAGE_TYPE, StorageType.LOCALFILE_HDFS.toString());
+    shuffleServerConf.set(RssBaseConf.RSS_STORAGE_BASE_PATH, Arrays.asList(tempDir.getAbsolutePath()));
+    shuffleServerConf.set(ShuffleServerConf.DISK_CAPACITY, 100L);
+    shuffleServerConf.setString(ShuffleServerConf.MULTISTORAGE_FALLBACK_STRATEGY_CLASS,
+        LocalStorageManagerFallbackStrategy.class.getCanonicalName());
+
+    StorageManager storageManager = StorageManagerFactory.getInstance().createStorageManager(shuffleServerConf);
+    String remoteStorage = "test";
+    String appId = "fallbackWrittenWhenMultiStorageManagerEnableTest";
+    storageManager.registerRemoteStorage(appId, new RemoteStorageInfo(remoteStorage));
+
+    ShuffleFlushManager flushManager = new ShuffleFlushManager(
+        shuffleServerConf,
+        "shuffle-server-id",
+        mockShuffleServer,
+        storageManager
+    );
+
+    // case1: normally written to local storage
+    ShuffleDataFlushEvent event = createShuffleDataFlushEvent(appId, 1, 1, 1, null, 100);
+    flushManager.addToFlushQueue(event);
+    Thread.sleep(1000);
+    assertTrue(event.getUnderStorage() instanceof LocalStorage);
+
+    // case2: huge event is written to cold storage directly
+    event = createShuffleDataFlushEvent(appId, 1, 1, 1, null, 100000);
+    flushManager.addToFlushQueue(event);
+    Thread.sleep(1000);
+    assertTrue(event.getUnderStorage() instanceof HdfsStorage);
+    assertEquals(0, event.getRetryTimes());
+
+    // case3: local disk is full or corrupted, fallback to HDFS
+    List<ShufflePartitionedBlock> blocks = Lists.newArrayList(
+        new ShufflePartitionedBlock(100000, 1000, 1, 1, 1L, null)
+    );
+    ShuffleDataFlushEvent bigEvent = new ShuffleDataFlushEvent(1, "1", 1, 1, 1, 100, blocks, null, null);
+    bigEvent.setUnderStorage(((MultiStorageManager)storageManager).getWarmStorageManager().selectStorage(event));
+    ((MultiStorageManager)storageManager).getWarmStorageManager().updateWriteMetrics(bigEvent, 0);
+
+    event = createShuffleDataFlushEvent(appId, 1, 1, 1, null, 100);
+    flushManager.addToFlushQueue(event);
+    Thread.sleep(1000);
+    assertTrue(event.getUnderStorage() instanceof HdfsStorage);
+    assertEquals(1, event.getRetryTimes());
+  }
+
   @Test
   public void processPendingEventsTest(@TempDir File tempDir) {
     try {
@@ -416,7 +526,7 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
           StorageManagerFactory.getInstance().createStorageManager(shuffleServerConf);
       ShuffleFlushManager manager =
           new ShuffleFlushManager(shuffleServerConf, "shuffleServerId", mockShuffleServer, storageManager);
-      ShuffleDataFlushEvent event = new ShuffleDataFlushEvent(1, "1", 1, 1,1, 100, null, null, null);
+      ShuffleDataFlushEvent event = new ShuffleDataFlushEvent(1, "1", 1, 1, 1, 100, null, null, null);
       assertEquals(0, manager.getPendingEventsSize());
       manager.addPendingEvents(event);
       Thread.sleep(1000);
@@ -424,9 +534,12 @@ public class ShuffleFlushManagerTest extends HdfsTestBase {
       do {
         Thread.sleep(1 * 1000);
       } while (manager.getEventNumInFlush() != 0);
+
       List<ShufflePartitionedBlock> blocks = Lists.newArrayList(new ShufflePartitionedBlock(100, 1000, 1, 1, 1L, null));
       ShuffleDataFlushEvent bigEvent = new ShuffleDataFlushEvent(1, "1", 1, 1, 1, 100, blocks, null, null);
+      bigEvent.setUnderStorage(storageManager.selectStorage(event));
       storageManager.updateWriteMetrics(bigEvent, 0);
+
       manager.addPendingEvents(event);
       manager.addPendingEvents(event);
       manager.addPendingEvents(event);
