@@ -18,32 +18,38 @@
 package org.apache.uniffle.server.buffer;
 
 import java.io.File;
+import java.time.Duration;
 import java.util.Arrays;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicLong;
 
 import com.google.common.collect.RangeMap;
-import com.google.common.io.Files;
+import org.awaitility.Awaitility;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
+import org.roaringbitmap.longlong.Roaring64NavigableMap;
 
 import org.apache.uniffle.common.ShuffleDataResult;
 import org.apache.uniffle.common.ShufflePartitionedData;
+import org.apache.uniffle.common.rpc.StatusCode;
 import org.apache.uniffle.common.util.Constants;
 import org.apache.uniffle.server.ShuffleFlushManager;
 import org.apache.uniffle.server.ShuffleServer;
 import org.apache.uniffle.server.ShuffleServerConf;
 import org.apache.uniffle.server.ShuffleServerMetrics;
 import org.apache.uniffle.server.ShuffleTaskManager;
-import org.apache.uniffle.server.StatusCode;
+import org.apache.uniffle.server.TestShuffleFlushManager;
 import org.apache.uniffle.server.storage.StorageManager;
 import org.apache.uniffle.server.storage.StorageManagerFactory;
 import org.apache.uniffle.storage.util.StorageType;
 
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.Mockito.mock;
@@ -54,12 +60,13 @@ import static org.mockito.Mockito.when;
 public class ShuffleBufferManagerTest extends BufferTestBase {
   private ShuffleBufferManager shuffleBufferManager;
   private ShuffleFlushManager mockShuffleFlushManager;
+  private ShuffleServer mockShuffleServer;
+  private ShuffleTaskManager mockShuffleTaskManager;
   private ShuffleServerConf conf;
 
   @BeforeEach
-  public void setUp() {
+  public void setUp(@TempDir File tmpDir) {
     conf = new ShuffleServerConf();
-    File tmpDir = Files.createTempDir();
     File dataDir = new File(tmpDir, "data");
     conf.setString(ShuffleServerConf.RSS_STORAGE_TYPE, StorageType.LOCALFILE.name());
     conf.set(ShuffleServerConf.RSS_STORAGE_BASE_PATH, Arrays.asList(dataDir.getAbsolutePath()));
@@ -68,6 +75,9 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
     conf.set(ShuffleServerConf.SERVER_MEMORY_SHUFFLE_HIGHWATERMARK_PERCENTAGE, 80.0);
     conf.setLong(ShuffleServerConf.DISK_CAPACITY, 1024L * 1024L * 1024L);
     mockShuffleFlushManager = mock(ShuffleFlushManager.class);
+    mockShuffleServer = mock(ShuffleServer.class);
+    mockShuffleTaskManager = mock(ShuffleTaskManager.class);
+    when(mockShuffleServer.getShuffleTaskManager()).thenReturn(mockShuffleTaskManager);
     shuffleBufferManager = new ShuffleBufferManager(conf, mockShuffleFlushManager);
   }
 
@@ -94,6 +104,49 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
     assertEquals(buffer, bufferPool.get(appId).get(shuffleId).get(0));
   }
 
+  @Test
+  public void getShuffleDataWithExpectedTaskIdsTest() {
+    String appId = "getShuffleDataWithExpectedTaskIdsTest";
+    shuffleBufferManager.registerBuffer(appId, 1, 0, 1);
+    ShufflePartitionedData spd1 = createData(0, 1, 68);
+    ShufflePartitionedData spd2 = createData(0, 2, 68);
+    ShufflePartitionedData spd3 = createData(0, 1, 68);
+    ShufflePartitionedData spd4 = createData(0, 3, 68);
+    shuffleBufferManager.cacheShuffleData(appId, 1, false, spd1);
+    shuffleBufferManager.cacheShuffleData(appId, 1, false, spd2);
+    shuffleBufferManager.cacheShuffleData(appId, 1, false, spd3);
+    shuffleBufferManager.cacheShuffleData(appId, 1, false, spd4);
+
+    /**
+     * case1: all blocks in cached and read multiple times
+     */
+    ShuffleDataResult result = shuffleBufferManager.getShuffleData(
+        appId,
+        1,
+        0,
+        Constants.INVALID_BLOCK_ID,
+        60,
+        Roaring64NavigableMap.bitmapOf(1)
+    );
+    assertEquals(1, result.getBufferSegments().size());
+    assertEquals(0, result.getBufferSegments().get(0).getOffset());
+    assertEquals(68, result.getBufferSegments().get(0).getLength());
+
+    // 2nd read
+    long lastBlockId = result.getBufferSegments().get(0).getBlockId();
+    result = shuffleBufferManager.getShuffleData(
+        appId,
+        1,
+        0,
+        lastBlockId,
+        60,
+        Roaring64NavigableMap.bitmapOf(1)
+    );
+    assertEquals(1, result.getBufferSegments().size());
+    assertEquals(0, result.getBufferSegments().get(0).getOffset());
+    assertEquals(68, result.getBufferSegments().get(0).getLength());
+  }
+
   @Test
   public void getShuffleDataTest() {
     String appId = "getShuffleDataTest";
@@ -133,7 +186,7 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
     assertEquals(1, bufferPool.get(appId).get(2).get(0).getInFlushBlockMap().size());
     assertEquals(0, bufferPool.get(appId).get(3).get(0).getBlocks().size());
     assertEquals(1, bufferPool.get(appId).get(3).get(0).getInFlushBlockMap().size());
-    // keep buffer whose size < low water mark
+    // keep buffer whose size < low watermark
     assertEquals(1, bufferPool.get(appId).get(4).get(0).getBlocks().size());
     // data in flush buffer now, it also can be got before flush finish
     sdr = shuffleBufferManager.getShuffleData(
@@ -251,7 +304,7 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
     sc = shuffleBufferManager.cacheShuffleData(appId, shuffleId, false, createData(0, 1));
     assertEquals(StatusCode.NO_BUFFER, sc);
 
-    // size won't be reduce which should be processed by flushManager, reset buffer size to 0
+    // size won't be reduced which should be processed by flushManager, reset buffer size to 0
     shuffleBufferManager.resetSize();
     shuffleBufferManager.removeBuffer(appId);
     assertEquals(startPartitionNum, (int) ShuffleServerMetrics.gaugeTotalPartitionNum.get());
@@ -286,9 +339,9 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
     // receive data with preAllocation
     shuffleBufferManager.cacheShuffleData(appId, shuffleId, true, createData(0, 16));
     assertEquals(48, shuffleBufferManager.getUsedMemory());
-    assertEquals(0, shuffleBufferManager.getPreAllocatedSize());
+    assertEquals(48, shuffleBufferManager.getPreAllocatedSize());
     // release memory
-    shuffleBufferManager.releaseMemory(48, false, false);
+    shuffleBufferManager.releaseMemory(48, false, true);
     assertEquals(0, shuffleBufferManager.getUsedMemory());
     assertEquals(0, shuffleBufferManager.getPreAllocatedSize());
     // receive data without preAllocation
@@ -317,12 +370,16 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
 
     // actual data size < spillThreshold, won't flush
     sc = shuffleBufferManager.cacheShuffleData(appId, shuffleId, true, createData(1, 16));
+    shuffleBufferManager.releasePreAllocatedSize(48);
     assertEquals(StatusCode.SUCCESS, sc);
     assertEquals(500, shuffleBufferManager.getUsedMemory());
     assertEquals(452, shuffleBufferManager.getPreAllocatedSize());
 
     // actual data size > highWaterMark, flush
     shuffleBufferManager.cacheShuffleData(appId, shuffleId, true, createData(0, 400));
+    shuffleBufferManager.releasePreAllocatedSize(432);
+    // trigger flush manually
+    shuffleBufferManager.flushIfNecessary();
     assertEquals(StatusCode.SUCCESS, sc);
     assertEquals(500, shuffleBufferManager.getUsedMemory());
     assertEquals(20, shuffleBufferManager.getPreAllocatedSize());
@@ -391,9 +448,73 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
   }
 
   @Test
-  public void flushSingleBufferTest() throws Exception {
+  public void flushSingleBufferForHugePartitionTest(@TempDir File tmpDir) throws Exception {
+    ShuffleServerConf shuffleConf = new ShuffleServerConf();
+    File dataDir = new File(tmpDir, "data");
+    shuffleConf.setString(ShuffleServerConf.RSS_STORAGE_TYPE, StorageType.LOCALFILE.name());
+    shuffleConf.set(ShuffleServerConf.RSS_STORAGE_BASE_PATH, Arrays.asList(dataDir.getAbsolutePath()));
+    shuffleConf.set(ShuffleServerConf.SERVER_MEMORY_SHUFFLE_LOWWATERMARK_PERCENTAGE, 20.0);
+    shuffleConf.set(ShuffleServerConf.SERVER_MEMORY_SHUFFLE_HIGHWATERMARK_PERCENTAGE, 80.0);
+    shuffleConf.setLong(ShuffleServerConf.DISK_CAPACITY, 1024L * 1024L * 1024L);
+    shuffleConf.set(ShuffleServerConf.SERVER_BUFFER_CAPACITY, 200L);
+    shuffleConf.set(ShuffleServerConf.HUGE_PARTITION_MEMORY_USAGE_LIMITATION_RATIO, 0.1);
+    shuffleConf.set(ShuffleServerConf.HUGE_PARTITION_SIZE_THRESHOLD, 100L);
+    shuffleConf.setSizeAsBytes(ShuffleServerConf.SINGLE_BUFFER_FLUSH_THRESHOLD, 64L);
+
+    ShuffleServer mockShuffleServer = mock(ShuffleServer.class);
+    StorageManager storageManager = StorageManagerFactory.getInstance().createStorageManager(shuffleConf);
+    ShuffleFlushManager shuffleFlushManager =
+        new ShuffleFlushManager(shuffleConf, "serverId", mockShuffleServer, storageManager);
+    shuffleBufferManager = new ShuffleBufferManager(shuffleConf, shuffleFlushManager);
+    ShuffleTaskManager shuffleTaskManager =
+        new ShuffleTaskManager(shuffleConf, shuffleFlushManager, shuffleBufferManager, storageManager);
+
+    when(mockShuffleServer
+        .getShuffleFlushManager())
+        .thenReturn(shuffleFlushManager);
+    when(mockShuffleServer
+        .getShuffleBufferManager())
+        .thenReturn(shuffleBufferManager);
+    when(mockShuffleServer
+        .getShuffleTaskManager())
+        .thenReturn(shuffleTaskManager);
+
+    String appId = "flushSingleBufferForHugePartitionTest_appId";
+    int shuffleId = 1;
+
+    // case1: its partition is not huge partition
+    shuffleBufferManager.registerBuffer(appId, shuffleId, 0, 0);
+    ShufflePartitionedData partitionedData = createData(0, 1);
+    shuffleTaskManager.cacheShuffleData(appId, shuffleId, false, partitionedData);
+    shuffleTaskManager.updateCachedBlockIds(appId, shuffleId, 0, partitionedData.getBlockList());
+    assertEquals(1 + 32, shuffleBufferManager.getUsedMemory());
+    long usedSize = shuffleTaskManager.getPartitionDataSize(appId, shuffleId, 0);
+    assertEquals(1 + 32, usedSize);
+    assertFalse(
+        shuffleBufferManager.limitHugePartition(appId, shuffleId, 0,
+            shuffleTaskManager.getPartitionDataSize(appId, shuffleId, 0)
+        )
+    );
+
+    // case2: its partition is huge partition, its buffer will be flushed to DISK directly
+    partitionedData = createData(0, 36);
+    shuffleTaskManager.cacheShuffleData(appId, shuffleId, false, partitionedData);
+    shuffleTaskManager.updateCachedBlockIds(appId, shuffleId, 0, partitionedData.getBlockList());
+    assertEquals(33 + 36 + 32, shuffleBufferManager.getUsedMemory());
+    assertTrue(
+        shuffleBufferManager.limitHugePartition(appId, shuffleId, 0,
+            shuffleTaskManager.getPartitionDataSize(appId, shuffleId, 0)
+        )
+    );
+    partitionedData = createData(0, 1);
+    shuffleTaskManager.cacheShuffleData(appId, shuffleId, false, partitionedData);
+    shuffleTaskManager.updateCachedBlockIds(appId, shuffleId, 0, partitionedData.getBlockList());
+    waitForFlush(shuffleFlushManager, appId, shuffleId, 3);
+  }
+
+  @Test
+  public void flushSingleBufferTest(@TempDir File tmpDir) throws Exception {
     ShuffleServerConf shuffleConf = new ShuffleServerConf();
-    File tmpDir = Files.createTempDir();
     File dataDir = new File(tmpDir, "data");
     shuffleConf.setString(ShuffleServerConf.RSS_STORAGE_TYPE, StorageType.LOCALFILE.name());
     shuffleConf.set(ShuffleServerConf.RSS_STORAGE_BASE_PATH, Arrays.asList(dataDir.getAbsolutePath()));
@@ -440,6 +561,54 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
     assertEquals(0, shuffleBufferManager.getInFlushSize());
   }
 
+  @Test
+  public void shuffleFlushThreshold() throws Exception {
+    ShuffleServerConf serverConf = new ShuffleServerConf();
+    serverConf.addAll(conf);
+    serverConf.set(ShuffleServerConf.SERVER_BUFFER_CAPACITY, 180L);
+    serverConf.set(ShuffleServerConf.SERVER_SHUFFLE_FLUSH_THRESHOLD, 64L);
+
+    StorageManager storageManager = StorageManagerFactory.getInstance().createStorageManager(conf);
+    TestShuffleFlushManager shuffleFlushManager = new TestShuffleFlushManager(conf,
+        "serverId", mockShuffleServer, storageManager);
+    shuffleBufferManager = new ShuffleBufferManager(serverConf, shuffleFlushManager);
+
+    String appId = "shuffleFlushTest";
+    int shuffleId = 0;
+    int smallShuffleId = 1;
+    int smallShuffleIdTwo = 2;
+
+    shuffleBufferManager.registerBuffer(appId, shuffleId, 0, 1);
+    shuffleBufferManager.registerBuffer(appId, shuffleId, 2, 3);
+    shuffleBufferManager.registerBuffer(appId, smallShuffleId, 0,1);
+    shuffleBufferManager.registerBuffer(appId, smallShuffleIdTwo,0, 1);
+    shuffleBufferManager.cacheShuffleData(appId, shuffleId, false, createData(0, 64));
+    assertEquals(96, shuffleBufferManager.getUsedMemory());
+    shuffleBufferManager.cacheShuffleData(appId, smallShuffleId, false, createData(0, 31));
+    assertEquals(96 + 63, shuffleBufferManager.getUsedMemory());
+    shuffleFlushManager.flush();
+    // small shuffle id is kept in memory
+    assertEquals(63, shuffleBufferManager.getUsedMemory());
+    assertEquals(0, shuffleBufferManager.getInFlushSize());
+
+    // more data will trigger the flush
+    shuffleBufferManager.cacheShuffleData(appId, smallShuffleId, false, createData(0, 31));
+    shuffleBufferManager.cacheShuffleData(appId, smallShuffleId, false, createData(0, 31));
+    assertEquals(63 * 3, shuffleBufferManager.getUsedMemory());
+    shuffleFlushManager.flush();
+    assertEquals(0, shuffleBufferManager.getUsedMemory());
+    assertEquals(0, shuffleBufferManager.getInFlushSize());
+
+    // all the small data in shuffle server, which could be extremely rare
+    shuffleBufferManager.cacheShuffleData(appId, shuffleId, false, createData(0, 22));
+    shuffleBufferManager.cacheShuffleData(appId, smallShuffleId, false, createData(0, 21));
+    shuffleBufferManager.cacheShuffleData(appId, smallShuffleIdTwo, false, createData(0, 20));
+    assertEquals(54 + 53 + 52, shuffleBufferManager.getUsedMemory());
+    shuffleFlushManager.flush();
+    assertEquals(52, shuffleBufferManager.getUsedMemory());
+    assertEquals(0, shuffleBufferManager.getInFlushSize());
+  }
+
   private void waitForFlush(ShuffleFlushManager shuffleFlushManager,
       String appId, int shuffleId, int expectedBlockNum) throws Exception {
     int retry = 0;
@@ -454,5 +623,10 @@ public class ShuffleBufferManagerTest extends BufferTestBase {
         fail("Flush data time out");
       }
     } while (committedCount < expectedBlockNum);
+
+    // Need to wait for `event.doCleanup` to be executed
+    // to ensure the correctness of subsequent checks of
+    // `shuffleBufferManager.getUsedMemory()` and `shuffleBufferManager.getInFlushSize()`.
+    Awaitility.await().atMost(Duration.ofSeconds(5)).until(() -> shuffleBufferManager.getUsedMemory() == 0);
   }
 }
